
local Map = Object:extend()

function Map:new(data)
  self.layers = data.layers or error("No layers found!")
  self.tiles = data.tiles or error("No tiles found!")
  self.collisionBoxes = {}
  self.spawn = nil -- gets set when there is a spawn tile on the loaded map
  self.showCollisionBoxes = false
  self:generate()
end


function Map:generate() -- this interate over each tile in each layer
  for _, layer in pairs(self.layers) do
    for x = 1, #layer.data[1] do
      for y = 1, #layer.data do
        
        -- Current Tile --
        local tile = self.tiles[layer.data[y][x]]

        if tile then
          -- Generate Collision Boxes --
          -- This creates the collision objects for the tiles, currently one box for each tile. This need to get optimised later when 
          -- you have a map editor ready. Then you can reduce the amount of collision boxes by connecting them with neighbor tiles.
          -- It looks into each layer on your map and filter the solid blocks. 
          self:generateCollisionBox(tile, x, y)

          -- Set Player Spawn --
          if tile.name == "Spawn" then
            self.spawn = {x = x*tile.width-tile.width, y = y*tile.height-tile.height}
          end
        end
        
        -- Setup canvas --
        -- This reduces the drawing operations each frame to only 1 for every layer instead of amountOfTiles*amountLayers(high amount)!
        -- Note that you need to refresh the canvas if you want to change something on the tile map during runtime.
        self:refreshCanvas(layer)
      
      end
    end
  end
end


function Map:refreshCanvas(layer)
  if not layer.canvas then layer.canvas = love.graphics.newCanvas(800, 600)
  else layer.canvas:clear() end
  love.graphics.setCanvas(layer.canvas)
    for x = 1, #layer.data[1] do
      for y = 1, #layer.data do
        local tile = self.tiles[layer.data[y][x]]
        if tile then
          love.graphics.setColor(unpack(tile.color))
          love.graphics.rectangle(layer.drawMode, x*tile.width-tile.width, y*tile.height-tile.height, tile.width, tile.height)
          love.graphics.setColor(255, 255, 255, 255)
        end
      end
    end
  love.graphics.setCanvas()
end


function Map:drawLayer(layer)
  love.graphics.draw(layer.canvas, 0, 0)
end


function Map:generateCollisionBox(tile, x, y)
  if not tile then error("No Tile given!") end
  if tile.solid then
    local box = {x = x*tile.width-tile.width, y = y*tile.height-tile.height, width = tile.width, height = tile.height}
    table.insert(self.collisionBoxes, box)
  end 
end


function Map:getCollisionBoxes()
  return self.collisionBoxes
end


function Map:getSpawn()
  if not self.spawn then error("No player spawn found!") end
  return self.spawn.x, self.spawn.y
end


function Map:toggleDebug()
  self.showCollisionBoxes = not self.showCollisionBoxes
end


function Map:draw()
  -- Map drawing should get optimised. You can draw each layer onto a spritebatch and draw it. This would improve the performance drastically.
  -- Especially on larger maps.
  -- Draw the layers --
  for k, layer in pairs(self.layers) do
    if layer.isVisible then self:drawLayer(layer) end
  end
  
  -- Draw the collision boxes --
  -- (just for debugging!) --
  if self.showCollisionBoxes then
    for k, box in pairs(self.collisionBoxes) do
      love.graphics.rectangle("line", box.x, box.y, box.width, box.height)
    end
  end
  
  -- Prints the about of collision boxes of the map --
  love.graphics.print("CollisionBoxes: "..#self.collisionBoxes, 10, 50)
end

return Map